gBypasserIndex=7 --be sure to update this if changes are made to the items array
gBypasserLastValue=1 --initially assume effects will be on
gSL1SetSelectorIndex=8 --index of virtual discrete value control

function remote_init(manufacturer, model)
	local items={
		{name="Keyboard", input="keyboard"},
		{name="Pitch Bend", input="value", min=0, max=16383},
		{name="Mod Wheel", input="value", min=0, max=127},
		{name="Damper Pedal", input="value", min=0, max=127},
		{name="Prev", input="button"},
		{name="Next", input="button"},

		--Virtual value triggered by Button 16 for use on effect enable switches. Allows toggling between Bypass and On.
		--When building remote maps, use either Bypasser or Button 16 for a given scope, but not both.
		{name="Bypasser", input="value", output="value", min=0, max=2},

		--Virtual value with a range of 0-11 that uses Buttons 9-15 to send the values 0-6 (one per button).
		--Designed for directly selecting Sets A through G on the Zvork Volt SL-1. Button 16 is left available for operating the on-off switch.
		{name="SL-1 Set Selector", input="value", min=0, max=11},

		--Buttons 1-8 work as pads and by default send MIDI notes as keyboard messages
		{name="Button 9", input="button", output="value"},
		{name="Button 10", input="button", output="value"},
		{name="Button 11", input="button", output="value"},
		{name="Button 12", input="button", output="value"},
		{name="Button 13", input="button", output="value"},
		{name="Button 14", input="button", output="value"},
		{name="Button 15", input="button", output="value"},
		{name="Button 16", input="button", output="value"},

		{name="Knob 1", input="delta"},
		{name="Knob 2", input="delta"},
		{name="Knob 3", input="delta"},
		{name="Knob 4", input="delta"},
		{name="Knob 5", input="delta"},
		{name="Knob 6", input="delta"},
		{name="Knob 7", input="delta"},
		{name="Knob 8", input="delta"},
		{name="Knob 9", input="delta"},
		{name="Knob 10", input="delta"},
		{name="Knob 11", input="delta"},
		{name="Knob 12", input="delta"},
		{name="Knob 13", input="delta"},
		{name="Knob 14", input="delta"},
		{name="Knob 15", input="delta"},
		{name="Knob 16", input="delta"},
	}
	remote.define_items(items)

	local inputs={
		{pattern="9? xx 00", name="Keyboard", value="0", note="x", velocity="64"}, --handles pads 1-8
		{pattern="<100x>? yy zz", name="Keyboard"},
		{pattern="E? xx yy", name="Pitch Bend", value="y*128 + x"},
		{pattern="B? 01 xx", name="Mod Wheel"},
		{pattern="B? 14 ?<???x>", name="Prev", value="x"},
		{pattern="B? 15 ?<???x>", name="Next", value="x"},

		{pattern="B? 16 ?<???x>", name="Button 9", value="x"},
		{pattern="B? 17 ?<???x>", name="Button 10", value="x"},
		{pattern="B? 18 ?<???x>", name="Button 11", value="x"},
		{pattern="B? 19 ?<???x>", name="Button 12", value="x"},
		{pattern="B? 1A ?<???x>", name="Button 13", value="x"},
		{pattern="B? 1B ?<???x>", name="Button 14", value="x"},
		{pattern="B? 1C ?<???x>", name="Button 15", value="x"},
		{pattern="B? 1D ?<???x>", name="Button 16", value="x"},

		{pattern="B? 07 <???y>x", name="Knob 1", value="x-16*y"},
		{pattern="B? 4A <???y>x", name="Knob 2", value="x-16*y"},
		{pattern="B? 47 <???y>x", name="Knob 3", value="x-16*y"},
		{pattern="B? 4C <???y>x", name="Knob 4", value="x-16*y"},
		{pattern="B? 4D <???y>x", name="Knob 5", value="x-16*y"},
		{pattern="B? 5D <???y>x", name="Knob 6", value="x-16*y"},
		{pattern="B? 49 <???y>x", name="Knob 7", value="x-16*y"},
		{pattern="B? 4B <???y>x", name="Knob 8", value="x-16*y"},
		{pattern="B? 72 <???y>x", name="Knob 9", value="x-16*y"},
		{pattern="B? 12 <???y>x", name="Knob 10", value="x-16*y"},
		{pattern="B? 13 <???y>x", name="Knob 11", value="x-16*y"},
		{pattern="B? 10 <???y>x", name="Knob 12", value="x-16*y"},
		{pattern="B? 11 <???y>x", name="Knob 13", value="x-16*y"},
		{pattern="B? 5B <???y>x", name="Knob 14", value="x-16*y"},
		{pattern="B? 4F <???y>x", name="Knob 15", value="x-16*y"},
		{pattern="B? 48 <???y>x", name="Knob 16", value="x-16*y"},
	}
	-- INPUTS THAT VARY DEPENDING ON MODEL --
	if model=="MiniLab" then
		table.insert(inputs, {pattern="B? 40 xx", name="Damper Pedal"}) --add pedal with standard polarity
	else
		table.insert(inputs, {pattern="B? 40 ?<???x>", name="Damper Pedal", value="(1-x)*127"}) --add pedal with reverse polarity
	end
	remote.define_auto_inputs(inputs)

	local outputs ={
		--[[ This type of output isn't serving to change anything on the MiniLab. Still need to find if there's a SysEx to change a button's lighted status.
		{name="Button 9", pattern="B0 16 xx", x="127*value"},
		]]
	}
	--remote.define_auto_outputs(outputs)
end

function remote_process_midi(event)
	ret=remote.match_midi("B? xx 40", event)
	if ret~=nil then
		if ret.x == 7
		or ret.x == 74
		or ret.x == 71
		or ret.x == 76
		or ret.x == 77
		or ret.x == 93
		or ret.x == 73
		or ret.x == 75
		or ret.x == 114
		or ret.x == 18
		or ret.x == 19
		or ret.x == 16
		or ret.x == 17
		or ret.x == 91
		or ret.x == 79
		or ret.x == 72 then  --for any encoder that sends a value of 40
			return true --handle extraneous encoder message by ignoring it
		end
	end
	ret=remote.match_midi("B? 1x ?<???1>", event) --process virtual value changes on button presses (buttons 9-16: x=6-13)
	if ret~=nil then
		--Handle "Bypasser" virtual value
		if ret.x == 13 then
			if gBypasserLastValue < 2 then
				gBypasserLastValue = 2 --switch to bypass mode if enabled switch is currently on or off
			else
				gBypasserLastValue = 1 --otherwise set switch to on
			end
			msg={ time_stamp=event.time_stamp, item=gBypasserIndex, value=gBypasserLastValue } --and send the new value to the item
			remote.handle_input(msg)
		end
		--Handle "SL-1 Set Selector" virtual value
		local buttonValue = ret.x - 6 --button 9 indicates a value of 0, 10 = 1, etc.
		if buttonValue >= 0 and buttonValue <= 6 then
			msg={ time_stamp=event.time_stamp, item=gSL1SetSelectorIndex, value=buttonValue } --create a MIDI message to send the button value to the Set Selector virtual value
			remote.handle_input(msg)
		end
		return false --we also want the auto input processing to handle these button presses normally
	end
	return false --incoming MIDI message hasn't been handled yet, forward to auto input handler
end

function remote_set_state(changed_items) --handle incoming changes sent by Reason
	for i,item_index in ipairs(changed_items) do
		if item_index==gBypasserIndex then --if the bypasser's value has changed within Reason (due to GUI interaction or locking to a device in a different state)
			gBypasserLastValue=remote.get_item_value(item_index) --change stored state to current state of attached device (scaled so it's guaranteed to be 0, 1 or 2)
		end
	end
end

function remote_probe()
	return {
		request="F0 7E 7F 06 01 F7",
		response="F0 7E ?? 06 02 00 20 6B 02 00 04 00 ?? ?? ?? ?? F7"
	}
end
